@@ -41,8 +41,11 @@ class Calibration(sc.prettyobj):
         sim          (Sim)  : the simulation to calibrate
         datafiles    (list) : list of datafile strings to calibrate to
         calib_pars   (dict) : a dictionary of the parameters to calibrate of the format dict(key1=[best, low, high])
+        genotype_pars(dict) : a dictionary of the genotype-specific parameters to calibrate of the format dict(genotype=dict(key1=[best, low, high]))
+        hiv_pars     (dict) : a dictionary of the hiv-specific parameters to calibrate of the format dict(key1=[best, low, high])
+        extra_sim_results (list) : list of result strings to store
         fit_args     (dict) : a dictionary of options that are passed to sim.compute_fit() to calculate the goodness-of-fit
-        par_samplers (dict) : an optional mapping from parameters to the Optuna sampler to use for choosing new points for each; by default, suggest_uniform
+        par_samplers (dict) : an optional mapping from parameters to the Optuna sampler to use for choosing new points for each; by default, suggest_float
         n_trials     (int)  : the number of trials per worker
         n_workers    (int)  : the number of parallel workers (default: maximum
         total_trials (int)  : if n_trials is not supplied, calculate by dividing this number by n_workers)
@@ -72,8 +75,8 @@ class Calibration(sc.prettyobj):
 
     '''
 
-    def __init__(self, sim, datafiles, calib_pars=None, genotype_pars=None, fit_args=None, par_samplers=None,
-                 n_trials=None, n_workers=None, total_trials=None, name=None, db_name=None,
+    def __init__(self, sim, datafiles, calib_pars=None, genotype_pars=None, hiv_pars=None, fit_args=None, extra_sim_results=None,
+                 par_samplers=None, n_trials=None, n_workers=None, total_trials=None, name=None, db_name=None,
                  keep_db=None, storage=None, rand_seed=None, label=None, die=False, verbose=True):
 
         import multiprocessing as mp # Import here since it's also slow
@@ -93,6 +96,8 @@ class Calibration(sc.prettyobj):
         self.sim            = sim
         self.calib_pars     = calib_pars
         self.genotype_pars  = genotype_pars
+        self.hiv_pars       = hiv_pars
+        self.extra_sim_results = extra_sim_results
         self.fit_args       = sc.mergedicts(fit_args)
         self.par_samplers   = sc.mergedicts(par_samplers)
         self.die            = die
@@ -105,7 +110,16 @@ class Calibration(sc.prettyobj):
             self.target_data.append(hpm.load_data(datafile))
 
         sim_results = sc.objdict()
-        age_result_keys = sc.objdict()
+
+        age_result_args = sc.objdict()
+        extra_sim_results = sc.objdict()
+
+        if self.extra_sim_results:
+            for extra_result in self.extra_sim_results:
+                extra_sim_results[extra_result] = sc.objdict()
+            self.extra_sim_results_keys = extra_sim_results.keys()
+        else:
+            self.extra_sim_results_keys = None
 
         # Go through each of the target keys and determine how we are going to get the results from sim
         for targ in self.target_data:
@@ -114,7 +128,7 @@ class Calibration(sc.prettyobj):
                 errormsg = f'Only support one set of targets per datafile, {len(targ_keys)} provided'
                 raise ValueError(errormsg)
             if 'age' in targ.columns:
-                age_result_keys[targ_keys[0]] = sc.objdict(
+                age_result_args[targ_keys[0]] = sc.objdict(
                     datafile=sc.dcp(targ),
                     compute_fit=True,
                 )
@@ -123,35 +137,46 @@ class Calibration(sc.prettyobj):
                     data=sc.dcp(targ)
                 )
 
-        ar = hpa.age_results(result_keys=age_result_keys)
+        ar = hpa.age_results(result_args=age_result_args)
         self.sim['analyzers'] += [ar]
+        if hiv_pars is not None:
+            self.sim['model_hiv'] = True # if calibrating HIV parameters, make sure model is running HIV
         self.sim.initialize()
         for rkey in sim_results.keys():
             sim_results[rkey].timepoints = sim.get_t(sim_results[rkey].data.year.unique()[0], return_date_format='str')[0]//sim.resfreq
             if 'weights' not in sim_results[rkey].data.columns:
                 sim_results[rkey].weights = np.ones(len(sim_results[rkey].data))
-        self.age_results_keys = age_result_keys.keys()
+        self.age_results_keys = age_result_args.keys()
         self.sim_results = sim_results
         self.sim_results_keys = sim_results.keys()
 
-        self.result_properties = sc.objdict()
+        self.result_args = sc.objdict()
         for rkey in self.age_results_keys + self.sim_results_keys:
-            self.result_properties[rkey] = sc.objdict()
-            self.result_properties[rkey].name = self.sim.results[rkey].name
-            self.result_properties[rkey].color = self.sim.results[rkey].color
+            self.result_args[rkey] = sc.objdict()
+            if 'hiv' in rkey:
+                self.result_args[rkey].name = self.sim.hivsim.results[rkey].name
+                self.result_args[rkey].color = self.sim.hivsim.results[rkey].color
+            else:
+                self.result_args[rkey].name = self.sim.results[rkey].name
+                self.result_args[rkey].color = self.sim.results[rkey].color
 
+        if self.extra_sim_results:
+            for rkey in self.extra_sim_results_keys:
+                self.result_args[rkey] = sc.objdict()
+                self.result_args[rkey].name = self.sim.results[rkey].name
+                self.result_args[rkey].color = self.sim.results[rkey].color
         # Temporarily store a filename
         self.tmp_filename = 'tmp_calibration_%05i.obj'
 
         return
 
 
-    def run_sim(self, calib_pars=None, genotype_pars=None, label=None, return_sim=False):
+    def run_sim(self, calib_pars=None, genotype_pars=None, hiv_pars=None, label=None, return_sim=False):
         ''' Create and run a simulation '''
         sim = self.sim.copy()
         if label: sim.label = label
 
-        new_pars = self.get_full_pars(sim=sim, calib_pars=calib_pars, genotype_pars=genotype_pars)
+        new_pars = self.get_full_pars(sim=sim, calib_pars=calib_pars, genotype_pars=genotype_pars, hiv_pars=hiv_pars)
         sim.update_pars(new_pars)
 
         # Run the sim
@@ -173,12 +198,23 @@ class Calibration(sc.prettyobj):
 
 
     @staticmethod
-    def get_full_pars(sim=None, calib_pars=None, genotype_pars=None):
-        ''' Make a full pardict from the subset of regular sim parameters and genotype parameters used in calibration'''
+    def get_full_pars(sim=None, calib_pars=None, genotype_pars=None, hiv_pars=None):
+        ''' Make a full pardict from the subset of regular sim parameters, genotype parameters, and hiv parameters used in calibration'''
 
         # Prepare the parameters
         if calib_pars is not None:
-            new_pars = {k:v for k,v in calib_pars.items() if k in sim.pars}
+            new_pars = {}
+            for name, par in calib_pars.items():
+                if isinstance(par, dict):
+                    simpar = sim.pars[name]
+                    for parkey, parval in par.items():
+                        simpar[parkey] = parval
+                    new_pars[name] = simpar
+                else:
+                    if name in sim.pars:
+                        new_pars[name] = par
+
+            # new_pars = {k:v for k,v in calib_pars.items() if k in sim.pars}
             if len(new_pars) != len(calib_pars):
                 extra = set(calib_pars.keys()) - set(new_pars.keys())
                 errormsg = f'The following parameters are not part of the sim, nor is a custom function specified to use them: {sc.strjoin(extra)}'
@@ -201,6 +237,23 @@ class Calibration(sc.prettyobj):
             all_genotype_pars.update(new_genotype_pars)
             new_pars['genotype_pars'] = all_genotype_pars
 
+        if hiv_pars is not None:
+            hiv_pars = {}
+            for name, par in hiv_pars.items():
+                if isinstance(par, dict):
+                    hivsimpar = sim.hivsim.pars['hiv_pars'][name]
+                    for parkey, parval in par.items():
+                        if isinstance(parval, dict):
+                            for parvalkey, parvalval in parval.items():
+                                hivsimpar[parkey][parvalkey] = parvalval
+                        else:
+                            hivsimpar[parkey] = parval
+                    hiv_pars[name] = hivsimpar
+                else:
+                    if name in sim.hivsim.pars['hiv_pars']:
+                        hiv_pars[name] = par
+            new_pars['hiv_pars'] = hiv_pars
+
         return new_pars
 
 
@@ -230,6 +283,7 @@ class Calibration(sc.prettyobj):
         # Initialize
         calib_pars = {}
         genotype_pars = sc.objdict()
+        hiv_pars = sc.objdict()
 
         # Deal with trial parameters
         if trial_pars is None:
@@ -254,16 +308,33 @@ class Calibration(sc.prettyobj):
                     this_genotype_pars[gpar] = trial_pars[f'{gname}_{gpar}']
             genotype_pars[gname] = this_genotype_pars
 
+        # Handle hiv sim parameters
+        if self.hiv_pars is not None:
+            for name, par in self.hiv_pars.items():
+                if isinstance(par, dict):
+                    hivsimpar = self.sim.hivsim.pars['hiv_pars'][name]
+                    for parkey in par.keys():
+                        hivsimpar[parkey] = trial_pars[f'{name}_{parkey}']
+                    hiv_pars[name] = hivsimpar
+                else:
+                    hiv_pars[name] = trial_pars[name]
+
         # Handle regular sim parameters
-        for pname in self.calib_pars.keys():
-            calib_pars[pname] = trial_pars[pname]
+        for name, par in self.calib_pars.items():
+            if isinstance(par, dict):
+                simpar = self.sim.pars[name]
+                for parkey in par.keys():
+                    simpar[parkey] = trial_pars[f'{name}_{parkey}']
+                calib_pars[name] = simpar
+            else:
+                calib_pars[name] = trial_pars[name]
 
         # Return
         if return_full:
-            all_pars = self.get_full_pars(sim=self.sim, calib_pars=calib_pars, genotype_pars=genotype_pars)
+            all_pars = self.get_full_pars(sim=self.sim, calib_pars=calib_pars, genotype_pars=genotype_pars, hiv_pars=hiv_pars)
             return all_pars
         else:
-            return pars, genotype_pars
+            return calib_pars, genotype_pars, hiv_pars
 
 
     def sim_to_sample_pars(self):
@@ -298,6 +369,20 @@ class Calibration(sc.prettyobj):
                             initial_pars[sampler_key] = par_highlowlist[0]
                             par_bounds[sampler_key] = np.array([par_highlowlist[1], par_highlowlist[2]])
 
+        # Convert hiv pars
+        if self.hiv_pars is not None:
+            for key, val in self.hiv_pars.items():
+                if isinstance(val, list):
+                    initial_pars[key] = val[0]
+                    par_bounds[key] = np.array([val[1], val[2]])
+                elif isinstance(val, dict):
+                    for parkey, par_highlowlist in val.items():
+                        sampler_key = key + '_' + parkey + '_'
+                        if isinstance(par_highlowlist, dict):
+                            par_highlowlist = par_highlowlist['value']
+                        initial_pars[sampler_key] = par_highlowlist[0]
+                        par_bounds[sampler_key] = np.array([par_highlowlist[1], par_highlowlist[2]])
+
         return initial_pars, par_bounds
 
 
@@ -316,7 +401,7 @@ class Calibration(sc.prettyobj):
                         errormsg = 'The requested sampler function is not found: ensure it is a valid attribute of an Optuna Trial object'
                         raise AttributeError(errormsg) from E
                 else:
-                    sampler_fn = trial.suggest_uniform
+                    sampler_fn = trial.suggest_float
                 if gname is not None:
                     sampler_key = gname + '_' + key
                 else:
@@ -324,13 +409,15 @@ class Calibration(sc.prettyobj):
                 pars[key] = sampler_fn(sampler_key, low, high)  # Sample from values within this range
 
             elif isinstance(val, dict):
-                sampler_fn = trial.suggest_uniform
+                sampler_fn = trial.suggest_float
                 pars[key] = dict()
                 for parkey, par_highlowlist in val.items():
                     if gname is not None:
                         sampler_key = gname + '_' + key + '_' + parkey
                     else:
                         sampler_key = key + '_' + parkey
+                    if isinstance(par_highlowlist, dict):
+                        par_highlowlist = par_highlowlist['value']
                     pars[key][parkey] = sampler_fn(sampler_key, par_highlowlist[1], par_highlowlist[2])
 
         return pars
@@ -345,12 +432,16 @@ class Calibration(sc.prettyobj):
                 genotype_pars[gname] = self.trial_to_sim_pars(pardict, trial, gname=gname)
         else:
             genotype_pars = None
+        if self.hiv_pars is not None:
+            hiv_pars = self.trial_to_sim_pars(self.hiv_pars, trial)
+        else:
+            hiv_pars = None
         if self.calib_pars is not None:
             calib_pars = self.trial_to_sim_pars(self.calib_pars, trial)
         else:
             calib_pars = None
 
-        sim = self.run_sim(calib_pars, genotype_pars, return_sim=True)
+        sim = self.run_sim(calib_pars, genotype_pars, hiv_pars, return_sim=True)
 
         # Compute fit for sim results and save sim results (TODO: THIS IS BY GENOTYPE FOR A SINGLE TIMEPOINT. GENERALIZE THIS)
         sim_results = sc.objdict()
@@ -363,9 +454,15 @@ class Calibration(sc.prettyobj):
             sim.fit += mismatch
             sim_results[rkey] = model_output
 
+        extra_sim_results = sc.objdict()
+        if self.extra_sim_results:
+            for rkey in self.extra_sim_results_keys:
+                model_output = sim.results[rkey]
+                extra_sim_results[rkey] = model_output
+
         # Store results in temporary files (TODO: consider alternatives)
         if save:
-            results = dict(sim=sim_results, analyzer=sim.get_analyzer().results)
+            results = dict(sim=sim_results, analyzer=sim.get_analyzer().results, extra_sim_results=extra_sim_results)
             filename = self.tmp_filename % trial.number
             sc.save(filename, results)
 
@@ -427,7 +524,7 @@ class Calibration(sc.prettyobj):
         return output
 
 
-    def calibrate(self, calib_pars=None, genotype_pars=None, verbose=True, load=True, tidyup=True, **kwargs):
+    def calibrate(self, calib_pars=None, genotype_pars=None, hiv_pars=None, verbose=True, load=True, tidyup=True, **kwargs):
         '''
         Actually perform calibration.
 
@@ -443,7 +540,9 @@ class Calibration(sc.prettyobj):
             self.calib_pars = calib_pars
         if genotype_pars is not None:
             self.genotype_pars = genotype_pars
-        if (self.calib_pars is None) and (self.genotype_pars is None):
+        if hiv_pars is not None:
+            self.hiv_pars = hiv_pars
+        if (self.calib_pars is None) and (self.genotype_pars is None) and (self.hiv_pars is None):
             errormsg = 'You must supply calibration parameters (calib_pars or genotype_pars) either when creating the calibration object or when calling calibrate().'
             raise ValueError(errormsg)
         self.run_args.update(kwargs) # Update optuna settings
@@ -474,6 +573,7 @@ class Calibration(sc.prettyobj):
                     results = sc.load(filename)
                     self.sim_results.append(results['sim'])
                     self.analyzer_results.append(results['analyzer'])
+                    self.extra_sim_results.append(results['extra_sim_results'])
                     if tidyup:
                         try:
                             os.remove(filename)
@@ -552,7 +652,7 @@ class Calibration(sc.prettyobj):
             return json
 
 
-    def plot(self, res_to_plot=None, fig_args=None, axis_args=None, data_args=None, do_save=None,
+    def plot(self, res_to_plot=None, fig_args=None, axis_args=None, data_args=None, show_args=None, do_save=None,
              fig_path=None, do_show=True, plot_type='sns.boxplot', **kwargs):
         '''
         Plot the calibration results
@@ -572,7 +672,7 @@ class Calibration(sc.prettyobj):
         if sc.isstring(plot_type) and plot_type.startswith('sns'):
             import seaborn as sns
             if plot_type.split('.')[1]=='boxplot':
-                extra_args=dict(boxprops=dict(alpha=.3))
+                extra_args=dict(boxprops=dict(alpha=.3), showfliers=False)
             else: extra_args = dict()
             plot_func = getattr(sns, plot_type.split('.')[1])
         else:
@@ -583,7 +683,8 @@ class Calibration(sc.prettyobj):
         fig_args = sc.mergedicts(dict(figsize=(12,8)), fig_args)
         axis_args = sc.mergedicts(dict(left=0.08, right=0.92, bottom=0.08, top=0.92), axis_args)
         d_args = sc.objdict(sc.mergedicts(dict(width=0.3, color='#000000', offset=0), data_args))
-        all_args = sc.mergedicts(fig_args, axis_args, d_args)
+        show_args = sc.objdict(sc.mergedicts(dict(show=dict(tight=True, maximize=False)), show_args))
+        all_args = sc.objdict(sc.mergedicts(fig_args, axis_args, d_args, show_args))
 
         # Pull out results to use
         analyzer_results = sc.dcp(self.analyzer_results)
@@ -639,13 +740,13 @@ class Calibration(sc.prettyobj):
                     unique_genotypes = thisdatadf.genotype.unique()
 
                     # Start making plot
-                    if 'total' not in resname and 'cancer' not in resname:
+                    if 'genotype' in resname:
                         for g in range(self.ng):
                             glabel = self.glabels[g].upper()
                             # Plot data
                             if glabel in unique_genotypes:
                                 ydata = np.array(thisdatadf[thisdatadf.genotype == glabel].value)
-                                ax.scatter(x, ydata, color=self.result_properties[resname].color[g], marker='s', label=f'Data - {glabel}')
+                                ax.scatter(x, ydata, color=self.result_args[resname].color[g], marker='s', label=f'Data - {glabel}')
 
                             # Construct a dataframe with things in the most logical order for plotting
                             for run_num, run in enumerate(analyzer_results):
@@ -660,7 +761,7 @@ class Calibration(sc.prettyobj):
                     else:
                         # Plot data
                         ydata = np.array(thisdatadf.value)
-                        ax.scatter(x, ydata, color=self.result_properties[resname].color, marker='s', label='Data')
+                        ax.scatter(x, ydata, color=self.result_args[resname].color, marker='s', label='Data')
 
                         # Construct a dataframe with things in the most logical order for plotting
                         for run_num, run in enumerate(analyzer_results):
@@ -669,11 +770,11 @@ class Calibration(sc.prettyobj):
 
                         # Plot model
                         modeldf = pd.DataFrame({'bins':bins, 'values':values})
-                        ax = plot_func(ax=ax, x='bins', y='values', data=modeldf, color=self.result_properties[resname].color, **extra_args)
+                        ax = plot_func(ax=ax, x='bins', y='values', data=modeldf, color=self.result_args[resname].color, **extra_args)
 
                     # Set title and labels
                     ax.set_xlabel('Age group')
-                    ax.set_title(self.result_properties[resname].name+', '+ date.replace('.0', ''))
+                    ax.set_title(self.result_args[resname].name+', '+ date.replace('.0', ''))
                     ax.legend()
                     ax.set_xticks(x, age_labels[resname])
                     plot_count += 1
@@ -685,7 +786,8 @@ class Calibration(sc.prettyobj):
                 values = []
                 thisdatadf = self.target_data[rn+sum(dates_per_result)][self.target_data[rn + sum(dates_per_result)].name == resname]
                 ydata = np.array(thisdatadf.value)
-                ax.scatter(x, ydata, color=self.result_properties[resname].color[0], marker='s', label='Data')
+                ax.scatter(x, ydata, color=pl.cm.Reds(0.95), marker='s', label='Data')
+
 
                 # Construct a dataframe with things in the most logical order for plotting
                 for run_num, run in enumerate(sim_results):
@@ -698,7 +800,7 @@ class Calibration(sc.prettyobj):
                 # Set title and labels
                 date = thisdatadf.year[0]
                 ax.set_xlabel('Genotype')
-                ax.set_title(self.result_properties[resname].name + ', ' + str(date))
+                ax.set_title(self.result_args[resname].name + ', ' + str(date))
                 ax.legend()
                 ax.set_xticks(x, self.glabels)
                 plot_count += 1
