@@ -17,20 +17,22 @@ __all__ = ['plot_sim', 'plot_scens', 'plot_scen_age_results', 'plot_result', 'pl
 #%% Plotting helper functions
 
 def handle_args(fig_args=None, plot_args=None, scatter_args=None, axis_args=None, fill_args=None,
-                legend_args=None, date_args=None, show_args=None, style_args=None, **kwargs):
+                bar_args=None, legend_args=None, date_args=None, show_args=None, style_args=None, contour_args=None, **kwargs):
     ''' Handle input arguments -- merge user input with defaults; see sim.plot for documentation '''
 
     # Set defaults
     defaults = sc.objdict()
-    defaults.fig     = sc.objdict(figsize=(10, 8), num=None)
-    defaults.plot    = sc.objdict(lw=1.5, alpha= 0.7)
-    defaults.scatter = sc.objdict(s=20, marker='s', alpha=0.7, zorder=1.75, datastride=1) # NB: 1.75 is above grid lines but below plots
-    defaults.axis    = sc.objdict(left=0.10, bottom=0.08, right=0.95, top=0.95, wspace=0.30, hspace=0.30)
-    defaults.fill    = sc.objdict(alpha=0.2)
-    defaults.legend  = sc.objdict(loc='best', frameon=False)
-    defaults.date    = sc.objdict(as_dates=True, dateformat=None, rotation=None, start=None, end=None)
-    defaults.show    = sc.objdict(data=True, ticks=True, interventions=True, legend=True, outer=False, tight=False, maximize=False)
-    defaults.style   = sc.objdict(style=None, dpi=None, font=None, fontsize=None, grid=None, facecolor=None) # Use HPVsim global defaults
+    defaults.fig        = sc.objdict(figsize=(10, 8), num=None)
+    defaults.plot       = sc.objdict(lw=1.5, alpha= 0.7)
+    defaults.scatter    = sc.objdict(s=20, marker='s', alpha=0.7, zorder=1.75, datastride=1) # NB: 1.75 is above grid lines but below plots
+    defaults.axis       = sc.objdict(left=0.10, bottom=0.08, right=0.95, top=0.95, wspace=0.30, hspace=0.30)
+    defaults.fill       = sc.objdict(alpha=0.2)
+    defaults.bar        = sc.objdict(width=0.15)
+    defaults.contours   = sc.objdict(levels=7, linewidths=0.5, colors='k')
+    defaults.legend     = sc.objdict(loc='best', frameon=False)
+    defaults.date       = sc.objdict(as_dates=True, dateformat=None, rotation=None, start=None, end=None)
+    defaults.show       = sc.objdict(data=True, ticks=True, interventions=True, legend=True, outer=False, tight=True, maximize=False)
+    defaults.style      = sc.objdict(style=None, dpi=None, font=None, fontsize=None, grid=None, facecolor=None) # Use HPVsim global defaults
 
     # Handle directly supplied kwargs
     for dkey,default in defaults.items():
@@ -49,15 +51,17 @@ def handle_args(fig_args=None, plot_args=None, scatter_args=None, axis_args=None
 
     # Merge arguments together
     args = sc.objdict()
-    args.fig     = sc.mergedicts(defaults.fig,     fig_args)
-    args.plot    = sc.mergedicts(defaults.plot,    plot_args)
-    args.scatter = sc.mergedicts(defaults.scatter, scatter_args)
-    args.axis    = sc.mergedicts(defaults.axis,    axis_args)
-    args.fill    = sc.mergedicts(defaults.fill,    fill_args)
-    args.legend  = sc.mergedicts(defaults.legend,  legend_args)
-    args.date    = sc.mergedicts(defaults.date,    date_args)
-    args.show    = sc.mergedicts(defaults.show,    show_args)
-    args.style   = sc.mergedicts(defaults.style,   style_args)
+    args.fig        = sc.mergedicts(defaults.fig,       fig_args)
+    args.plot       = sc.mergedicts(defaults.plot,      plot_args)
+    args.scatter    = sc.mergedicts(defaults.scatter,   scatter_args)
+    args.axis       = sc.mergedicts(defaults.axis,      axis_args)
+    args.fill       = sc.mergedicts(defaults.fill,      fill_args)
+    args.bar        = sc.mergedicts(defaults.bar,       bar_args)
+    args.contours   = sc.mergedicts(defaults.contours, contour_args)
+    args.legend     = sc.mergedicts(defaults.legend,    legend_args)
+    args.date       = sc.mergedicts(defaults.date,      date_args)
+    args.show       = sc.mergedicts(defaults.show,      show_args)
+    args.style      = sc.mergedicts(defaults.style,     style_args)
 
     # Handle potential rcParams keys
     keys = list(kwargs.keys())
@@ -130,36 +134,97 @@ def handle_to_plot(kind, to_plot, n_cols, sim, check_ready=True):
         errormsg = 'Cannot plot since results are not ready yet -- did you run the sim?'
         raise RuntimeError(errormsg)
 
-    # If it matches a result key, convert to a list
-    reskeys = sim.result_keys('total')
-    genkeys = sim.result_keys('genotype')
-    allkeys = reskeys + genkeys
-    if to_plot in allkeys:
-        to_plot = sc.tolist(to_plot)
+    # Define allowable choices for plotting - default plot type depends on result type
+    allkeys = sim.result_keys('all')
+    time_series_keys = sim.result_keys('total')+sim.result_keys('genotype')+sim.result_keys('sex')+sim.result_keys('type_dist')
+    age_dist_keys = sim.result_keys('age')
+    type_dist_keys = ['type_dist']
+    valid_keys = allkeys+type_dist_keys
+    def check_plot_type(which):
+        if which in time_series_keys: return 'time_series'
+        elif which in age_dist_keys: return 'age_dist'
+        elif which in type_dist_keys: return 'type_dist'
+        else:
+            raise ValueError(f'Plot type of {which} not understood.')
+
+    # analyzer_keys = [a.label for a in sim.get_analyzers()] # Defaults = whatever analyzer.plot() gives
+    n_extra_plots = 0 # Keep track of the number of extra plots from analyzers
+
+    # If to_plot is a single valid key, turn it into a list
+    if to_plot in valid_keys: to_plot = sc.tolist(to_plot)
+    if isinstance(to_plot, hpd.plot_args): to_plot = sc.tolist(to_plot)
 
     # If not specified or specified as another string, load defaults
     if to_plot is None or isinstance(to_plot, str):
         to_plot = hpd.get_default_plots(which=to_plot, kind=kind, sim=sim)
 
-    # If a list of keys has been supplied or constructed
+    # If it's a dictionary, translate it to a list but store the names
+    names = None
+    if isinstance(to_plot, dict):
+        to_plot_orig = to_plot # Hold onto original
+        names = [k for k in to_plot.keys()]
+        to_plot = [k for k in to_plot.values()]
+
+    # Validate list
     if isinstance(to_plot, list):
-        to_plot_list = to_plot # Store separately
-        to_plot = sc.odict() # Create the dict
+        to_plot_orig = to_plot[:] # Hold onto original for the moment
+        to_plot = sc.autolist()
         invalid = sc.autolist()
-        for reskey in to_plot_list:
-            if reskey in allkeys:
-                name = sim.results[reskey].name
-                to_plot[name] = [reskey] # Use the result name as the key and the reskey as the value
-            else:
-                invalid += reskey
+
+        # Loop over items in list and validate them
+        for rn,reskey in enumerate(to_plot_orig):
+
+            # If it's a string, we construct default plot args by checking what kind of result it is
+            if isinstance(reskey, str):
+                if reskey in valid_keys:
+                    if names is not None: name = names[rn]
+                    else:
+                        if reskey in allkeys:
+                            name = sim.results[reskey].name
+                        elif reskey == 'type_dist':
+                            name = 'HPV type distribution'
+                    if reskey in time_series_keys:
+                        to_plot += hpd.plot_args(reskey, name=name, plot_type='time_series')
+                    elif reskey in age_dist_keys:
+                        to_plot += hpd.plot_args(reskey, name=name, plot_type='age_dist', year=sim.results['year'][-1])
+                    elif reskey in type_dist_keys:
+                        to_plot += hpd.plot_args(reskey, name=name, plot_type='type_dist', year=sim.results['year'][-1])
+                else:
+                    invalid += reskey
+
+            # If it's plot args, we validate the years and set defaults
+            elif isinstance(reskey, hpd.plot_args):
+                if reskey.year == 'last': reskey.year = sim.results['year'][-1]
+                if reskey.plot_type is None: # Add sensible defaults if not supplied
+                    if reskey.keys[0] in age_dist_keys:
+                        reskey.plot_type = 'age_dist'
+                    elif reskey.keys[0] in type_dist_keys:
+                        reskey.plot_type = 'type_dist'
+                to_plot += reskey
+
+            # If it's a list, we ned to choose a single plot type
+            elif isinstance(reskey, list):
+                if names is not None: name = names[rn]
+                else: name = sim.results[reskey[0]].name # Use the name of the first result
+                plot_types = [check_plot_type(rkey) for rkey in reskey]
+                if 'time_series' in plot_types: # If no other info is provided, assume we want to plot them all as time series
+                    plot_type = 'time_series'
+                    year=None
+                elif 'age_dist' in plot_types:
+                    plot_type = 'age_dist'
+                    year = sim.results['year'][-1]
+                else:
+                    plot_type = 'type_dist'
+                    year = sim.results['year'][-1]
+                to_plot += hpd.plot_args(reskey, name=name, plot_type=plot_type, year=year)
+
+        # Raise an error if there are any invalid keys
         if len(invalid):
-            errormsg = f'The following key(s) are invalid:\n{sc.strjoin(invalid)}\n\nValid main keys are:\n{sc.strjoin(reskeys)}\n\nValid genotype keys are:\n{sc.strjoin(genkeys)}'
+            errormsg = f'The following key(s) are invalid:\n{sc.strjoin(invalid)}\n\nValid keys are:\n{sc.strjoin(valid_keys)}.'
             raise sc.KeyNotFoundError(errormsg)
 
-    to_plot = sc.odict(sc.dcp(to_plot)) # In case it's supplied as a dict
-
-    # Handle rows and columns -- assume 5 is the most rows we would want
-    n_plots = len(to_plot)
+    # Get total number of plots and calculate rows and columns
+    n_plots = len(to_plot) + n_extra_plots
     if n_cols is None:
         max_rows = 5 # Assumption -- if desired, the user can override this by setting n_cols manually
         n_cols = int((n_plots-1)//max_rows + 1) # This gives 1 column for 1-4, 2 for 5-8, etc.
@@ -334,74 +399,171 @@ def set_line_options(input_args, reskey, resnum, default):
 
 
 
-#%% Core plotting functions
+#%% Individual plotting functions to create particular plots
+def plot_time_series(ax, sim, reskey, resnum, args, colors=None, labels=None, plot_burnin=False):
+    ''' Plot time series data, i.e. the usual contents of sim.results '''
 
-def plot_sim(to_plot=None, sim=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
-         scatter_args=None, axis_args=None, fill_args=None, legend_args=None, date_args=None,
-         show_args=None, style_args=None, n_cols=None, grid=True, commaticks=True,
-         setylim=True, log_scale=False, colors=None, labels=None, do_show=None, sep_figs=False,
-         fig=None, ax=None, plot_burnin=False, **kwargs):
+    # Initialize some variables
+    bi = 0 if plot_burnin else int(sim['burnin'])
+    total_keys = sim.result_keys('total')
+    sex_keys = sim.result_keys('sex')
+    genotype_keys = sim.result_keys('genotype')
+    res_t = sim.results['year'][bi:]
+    res = sim.results[reskey]
+    ax.set_xlabel('Year')
+    ax.set_ylabel('Value')
+
+    # The exact plotting call depends on what kind of core result key we're dealing with
+    # Simplest case: it's a total result, i.e. not disagreggated by genotype or sex
+    if reskey in total_keys:
+        color = set_line_options(colors, reskey, resnum, res.color)  # Choose the color
+        label = set_line_options(labels, reskey, resnum, res.name)  # Choose the label
+        ax.plot(res_t, res.values[bi:], label=label, **args.plot, c=color)  # Plot result
+
+    elif reskey in sex_keys:
+        n_sexes = 2
+        sex_colors = ['#4679A2', '#A24679']
+        sex_labels = ['males', 'females']
+        for sex in range(n_sexes):
+            # Colors and labels
+            v_color = sex_colors[sex]
+            v_label = sex_labels[sex]  # TODO this should also come from the sim
+            color = set_line_options(colors, reskey, resnum, v_color)  # Choose the color
+            label = set_line_options(labels, reskey, resnum, res.name)  # Choose the label
+            if label:   label += f' - {v_label}'
+            else:       label = v_label
+            ax.plot(res_t, res.values[sex, bi:], label=label, **args.plot, c=color)  # Plot result
+
+    elif reskey in genotype_keys:
+        ng = sim['n_genotypes']
+        g_colors = sc.gridcolors(ng)
+        for genotype in range(ng):
+            # Colors and labels
+            g_color = g_colors[genotype]
+            geno_obj = sim['genotypes'][genotype]
+            if sc.isnumber(geno_obj):  # TODO: figure out why this is sometimes an int and sometimes an obj
+                v_label = str(geno_obj)
+            elif sc.isstring(geno_obj):
+                v_label = geno_obj
+            else:
+                v_label = geno_obj.label
+            color = set_line_options(colors, reskey, resnum, g_color)  # Choose the color
+            label = set_line_options(labels, reskey, resnum, res.name)  # Choose the label
+            if label:
+                label += f' - {v_label}'
+            else:
+                label = v_label
+            ax.plot(res_t, res.values[genotype, bi:], label=label, **args.plot, c=color)  # Plot result
+
+    else:
+        raise ValueError(f'Result {reskey} not understood.')
+
+    return ax, color
+
+
+def plot_type_bars(sim, ax, date, args):
+    '''
+    Plot HPV types by cytology
+    '''
+
+    idx = sc.findinds(sim.res_yearvec, date)[0]
+    labels = sc.autolist()
+    resdict = sc.objdict()
+    for rkey in sim.result_keys('type_dist'):
+        labels += sim.results[rkey].name
+        resdict[rkey] = sim.results[rkey][:,idx]
+    g_labels = sim['genotypes']
+
+    # Grouped bar plot with n_groups bars (one for each state) and ng bars per group
+    n_bars_per_group = sim['n_genotypes']
+    n_groups = len(resdict)
+    x = np.arange(n_groups)
+    width = args.bar.width
+
+    # Set position of bar on x axis
+    xpositions = [x]
+    for group_no in range(1, n_bars_per_group):
+        xpositions.append([xi + width for xi in xpositions[-1]])
+
+    # Plot bars
+    for bar_no in range(n_bars_per_group):
+        ydata = [resdict[k][bar_no] for k in range(n_groups)]  # Have to rearrange
+        ax.bar(xpositions[bar_no], ydata, **args.bar, label=g_labels[bar_no])
+
+    # Add xticks on the middle of the group bars
+    ax.set_xticks([r + width for r in range(len(x))], labels)
+    ax.set_xlabel('Health state')
+    ax.set_ylabel('Value')
+    
+    return ax
+
+
+def plot_age_dist(sim, ax, reskey, date, args):
+    '''
+    Function to plot a single age result for a single date. Requires an axis as
+    input and will generally be called by a helper function rather than directly.
+    '''
+    idx = sc.findinds(sim.res_yearvec, date)[0]
+    res = sim.results[reskey]
+    label = res.name.replace('by age', '')
+    x = sim['age_bins'][:-1]
+    ax.plot(x, res.values[:,idx], color=res.color, **args.plot, label=label)
+    ax.set_xlabel('Age')
+    ax.set_ylabel('Value')
+    return ax
+
+
+
+#%% Core plotting functions that unite the individual plotting functions to create figures for sims, scenarios, multisims, etc
+
+def plot_sim(to_plot=None, sim=None, fig=None, ax=None, do_save=None, fig_path=None,
+             fig_args=None, plot_args=None, scatter_args=None, axis_args=None, fill_args=None,
+             legend_args=None, date_args=None, show_args=None, style_args=None, n_cols=None,
+             grid=True, commaticks=True, setylim=True, log_scale=False, colors=None, labels=None,
+             do_show=None, sep_figs=False, plot_burnin=False, **kwargs):
     ''' Plot the results of a single simulation -- see Sim.plot() for documentation '''
 
     # Handle inputs
     args = handle_args(fig_args=fig_args, plot_args=plot_args, scatter_args=scatter_args, axis_args=axis_args, fill_args=fill_args,
                        legend_args=legend_args, show_args=show_args, date_args=date_args, style_args=style_args, **kwargs)
-    to_plot, n_cols, n_rows = handle_to_plot('sim', to_plot, n_cols, sim=sim)
+    to_plot, n_cols, n_rows = handle_to_plot('sim', to_plot, n_cols, sim)
 
     # Do the plotting
-    bi = 0 if plot_burnin else int(sim['burnin'])
     with hpo.with_style(args.style):
+
+        # Create the figures
         fig, figs = create_figs(args, sep_figs, fig, ax)
-        total_keys = sim.result_keys('total') # Consider a more robust way to do this
-        sex_keys = sim.result_keys('by_sex')
-        for pnum,title,keylabels in to_plot.enumitems():
-            ax = create_subplots(figs, fig, ax, n_rows, n_cols, pnum, args.fig, sep_figs, log_scale, title)
-            for resnum,reskey in enumerate(keylabels):
-                res_t = sim.results['year'][bi:]
-                res = sim.results[reskey]
-                if reskey in total_keys:
-                    color = set_line_options(colors, reskey, resnum, res.color)  # Choose the color
-                    label = set_line_options(labels, reskey, resnum, res.name)  # Choose the label
-                    ax.plot(res_t, res.values[bi:], label=label, **args.plot, c=color)  # Plot result
-                elif reskey in sex_keys:
-                    n_sexes = 2
-                    sex_colors = ['#4679A2', '#A24679']
-                    sex_labels = ['males', 'females']
-                    for sex in range(n_sexes):
-                        # Colors and labels
-                        v_color = sex_colors[sex]
-                        v_label = sex_labels[sex]  # TODO this should also come from the sim
-                        color = set_line_options(colors, reskey, resnum, v_color)  # Choose the color
-                        label = set_line_options(labels, reskey, resnum, res.name)  # Choose the label
-                        if label:
-                            label += f' - {v_label}'
-                        else:
-                            label = v_label
-                        ax.plot(res_t, res.values[sex, bi:], label=label, **args.plot, c=color)  # Plot result
-                else:
-                    ng = sim['n_genotypes']
-                    for genotype in range(ng):
-                        # Colors and labels
-                        v_color = res.color[genotype]
-                        geno_obj = sim['genotypes'][genotype]
-                        if sc.isnumber(geno_obj): # TODO: figure out why this is sometimes an int and sometimes an obj
-                            v_label = str(geno_obj)
-                        elif sc.isstring(geno_obj):
-                            v_label = geno_obj
-                        else:
-                            v_label = geno_obj.label
-                        color = set_line_options(colors, reskey, resnum, v_color)  # Choose the color
-                        label = set_line_options(labels, reskey, resnum, res.name)  # Choose the label
-                        if label: label += f' - {v_label}'
-                        else:     label = v_label
-                        ax.plot(res_t, res.values[genotype,bi:], label=label, **args.plot, c=color)  # Plot result
 
-                if args.show['data']:
-                    plot_data(sim, ax, reskey, args.scatter, color=color)  # Plot the data
+        # Determine whether to share x axis
+        do_sharex = False
+        plot_types = [tp.plot_type for tp in to_plot]
+        if len(set(plot_types))==1: do_sharex = True
+
+        # Iterate through to_plot to figure out what to plot & how to plot it
+        for pnum,plot_arg in enumerate(to_plot):
+            sharex = ax if do_sharex else None
+            title = plot_arg.name
+            plot_type = plot_arg.plot_type
+            ax = create_subplots(figs, fig, sharex, n_rows, n_cols, pnum, args.fig, sep_figs, log_scale, title)
+
+            if plot_type == 'time_series':
+                for resnum,reskey in enumerate(plot_arg.keys):
+                    ax, color = plot_time_series(ax, sim, reskey, resnum, args, labels=labels, colors=colors, plot_burnin=plot_burnin)
+                    if args.show['data']:
+                        plot_data(sim, ax, reskey, args.scatter, color=color)  # Plot the data
+                title_grid_legend(ax, title, grid, commaticks, setylim, args.legend, args.show)
+
+            elif plot_type == 'type_dist':
+                ax = plot_type_bars(sim, ax, plot_arg.year, args)
+                title = f'{title}, {int(plot_arg.year)}'
+                title_grid_legend(ax, title, grid, commaticks, setylim, args.legend, args.show)
+
+            elif plot_type == 'age_dist':
+                for resnum,reskey in enumerate(plot_arg.keys):
+                    ax = plot_age_dist(sim, ax, reskey, plot_arg.year, args)
+                title = f'{title}, {int(plot_arg.year)}'
+                title_grid_legend(ax, title, grid, commaticks, setylim, args.legend, args.show)
 
-            # if args.show['interventions']:
-            #     plot_interventions(sim, ax) # Plot the interventions
-            title_grid_legend(ax, title, grid, commaticks, setylim, args.legend, args.show) # Configure the title, grid, and legend
 
         output = tidy_up(fig, figs, do_save, fig_path, do_show, args)
 
@@ -424,9 +586,10 @@ def plot_scens(to_plot=None, scens=None, do_save=None, fig_path=None, fig_args=N
     with hpo.with_style(args.style):
         fig, figs = create_figs(args, sep_figs, fig, ax)
         default_colors = sc.gridcolors(ncolors=len(scens.sims))
-        for pnum,title,reskeys in to_plot.enumitems():
+        for pnum,plot_arg in enumerate(to_plot):
+            title = plot_arg.name
             ax = create_subplots(figs, fig, ax, n_rows, n_cols, pnum, args.fig, sep_figs, log_scale, title)
-            reskeys = sc.promotetolist(reskeys) # In case it's a string
+            reskeys = sc.promotetolist(plot_arg.keys) # In case it's a string
             for reskey in reskeys:
                 res_t = scens.res_yearvec
                 resdata = scens.results[reskey]
@@ -434,7 +597,7 @@ def plot_scens(to_plot=None, scens=None, do_save=None, fig_path=None, fig_args=N
                     sim = scens.sims[scenkey][0] # Pull out the first sim in the list for this scenario
                     bi = 0 if plot_burnin else int(sim['burnin'])
                     genotypekeys = sim.result_keys('genotype')
-                    sexkeys = sim.result_keys('by_sex')
+                    sexkeys = sim.result_keys('sex')
                     if reskey in genotypekeys:
                         ng = sim['n_genotypes']
                         genotype_colors = sc.gridcolors(ng)
@@ -592,6 +755,84 @@ def plot_result(key, sim=None, fig_args=None, plot_args=None, axis_args=None, sc
     return tidy_up(fig, figs, sep_figs, do_save, fig_path, do_show, args)
 
 
+def plot_heatmap(sweep, xx, yy, x=None, y=None, xi=None, yi=None, to_plot=None, xpar=None, ypar=None, npts=None,
+                 zscales=1, add_contours=True, contour_args=None, cmap='plasma',
+                 fig_args=None, plot_args=None, axis_args=None, legend_args=None, show_args=None, style_args=None,
+                 fig=None, ax=None, do_save=None, do_show=None, fig_path=None):
+    '''
+    Plot heatmaps
+    '''
+
+    # Handle inputs
+    axis_args = sc.mergedicts(axis_args, dict(wspace=0.05, right=0.9)) # Set right margin to be smaller
+    args = handle_args(fig_args=fig_args, plot_args=plot_args, axis_args=axis_args, contour_args=contour_args,
+                       legend_args=legend_args, show_args=show_args, style_args=style_args)
+
+    # Figure out axes
+    n_plots = len(to_plot)
+    n_rows, n_cols = sc.get_rows_cols(n_plots)
+    n_cols_true = 2*n_cols # Add one more columns per column, for storing the heatmap colorbars
+    width_ratios = [20, 1]*n_cols
+    npts = npts or 100
+    scale = 0.08
+
+    # Handle scale factors
+    if sc.isnumber(zscales): zscales = [zscales]*n_plots
+
+    # Plot
+    with hpo.with_style(args.style):
+        fig = pl.figure(**args.fig)
+        gs = fig.add_gridspec(n_rows, n_cols_true, width_ratios=width_ratios)
+        pl.subplots_adjust(**args.axis)
+        pn, coln = 0, 0
+
+        for rn in range(n_rows):
+            for cn in range(0,n_cols*2,2): # Increment by two columns each time
+
+                # Deal with precision
+                zscale = zscales[pn]
+                precision = zscale/10
+                rounding_factor = -int(np.log10(precision))
+
+                res_to_plot = to_plot[pn]
+                z = np.array(sweep.resdf[res_to_plot]) / zscale
+                z_min = min(z)
+                z_max = max(z)
+                zz = sc.gauss2d(x, y, z, xi, yi, scale=scale, xscale=1, yscale=1, grid=True)
+                scolors = sc.vectocolor(z, cmap=cmap, minval=z_min, maxval=z_max)
+
+                # Plot heatmap
+                axa = fig.add_subplot(gs[rn, cn])
+                ima = axa.contourf(xx, yy, zz, cmap=cmap) #, levels=np.linspace(z_min, z_max, npts))
+
+                # Optionally add scatter
+                if (x is not None) and (y is not None) and (scolors is not None):
+                    axa.scatter(x, y, marker='o', c=scolors, edgecolor=[0.3] * 3, s=50, linewidth=0.1, alpha=0.5)
+                if add_contours:
+                    axa.contour(xx, yy, zz, **args.contours)
+                axa.set_xlabel(xpar)
+                axa.set_ylabel(ypar)
+
+                # Get the right information in the title
+                title = sweep.res_labels[res_to_plot] # Result label
+                title += f', {int(sweep.from_year)}-{int(sweep.to_year)}'
+                if zscale != 1:
+                    if zscale == 100:           title += ' (hundreds)'
+                    elif zscale == 1000:        title += ' (thousands)'
+                    elif zscale == 10000:       title += ' (0000s)'
+                    elif zscale == 100000:      title += ' (00000s)'
+                    elif zscale == 1000000:     title += ' (M)'
+                axa.set_title(title)
+
+                # Colorbar
+                axc = fig.add_subplot(gs[rn, cn+1])
+                pl.colorbar(ima, cax=axc)
+
+                pn += 1
+
+    return tidy_up(fig, do_save, fig_path, do_show, args)
+
+
 # def plot_compare(df, log_scale=True, fig_args=None, axis_args=None, style_args=None, grid=False,
 #                  commaticks=True, setylim=True, color=None, label=None, fig=None,
 #                  do_save=None, do_show=None, fig_path=None, **kwargs):
