@@ -40,6 +40,7 @@ def make_people(sim, popdict=None, reset=False, verbose=None, use_age_data=True,
     n_agents = int(sim['n_agents']) # Shorten
     total_pop = None # Optionally created but always returned
     pop_trend = None # Populated later if location is specified
+    pop_age_trend = None
 
     if verbose is None:
         verbose = sim['verbose']
@@ -62,21 +63,20 @@ def make_people(sim, popdict=None, reset=False, verbose=None, use_age_data=True,
         # Other demographic data like mortality and fertility are also available by
         # country, but these are loaded directly into the sim since they are not
         # stored as part of the people.
-        age_data = hpd.default_age_data
         location = sim['location']
-        if location is not None:
-            if sim['verbose']:
-                print(f'Loading location-specific data for "{location}"')
-            if use_age_data:
-                try:
-                    age_data  = hpdata.get_age_distribution(location, year=sim['start'])
-                    pop_trend = hpdata.get_total_pop(location)
-                    total_pop = sum(age_data[:,2]) # Return the total population
-                except ValueError as E:
-                    warnmsg = f'Could not load age data for requested location "{location}" ({str(E)}), using default'
-                    hpm.warn(warnmsg)
+        if sim['verbose']:
+            print(f'Loading location-specific data for "{location}"')
+        if use_age_data:
+            try:
+                age_data = hpdata.get_age_distribution(location, year=sim['start'])
+                pop_trend = hpdata.get_total_pop(location)
+                total_pop = sum(age_data[:, 2])  # Return the total population
+                pop_age_trend = hpdata.get_age_distribution_over_time(location)
+            except ValueError as E:
+                warnmsg = f'Could not load age data for requested location "{location}" ({str(E)})'
+                hpm.warn(warnmsg, die=True)
 
-        uids, sexes, debuts, partners = set_static(n_agents, pars=sim.pars, sex_ratio=sex_ratio)
+        uids, sexes, debuts, rel_sev, partners = set_static(n_agents, pars=sim.pars, sex_ratio=sex_ratio)
 
         # Set ages, rounding to nearest timestep if requested
         age_data_min   = age_data[:,0]
@@ -96,6 +96,7 @@ def make_people(sim, popdict=None, reset=False, verbose=None, use_age_data=True,
         popdict['age'] = ages
         popdict['sex'] = sexes
         popdict['debut'] = debuts
+        popdict['rel_sev'] = rel_sev
         popdict['partners'] = partners
 
         is_active = ages > debuts
@@ -123,7 +124,7 @@ def make_people(sim, popdict=None, reset=False, verbose=None, use_age_data=True,
 
     # Do minimal validation and create the people
     validate_popdict(popdict, sim.pars, verbose=verbose)
-    people = hpppl.People(sim.pars, pop_trend=pop_trend, **popdict) # List for storing the people
+    people = hpppl.People(sim.pars, pop_trend=pop_trend, pop_age_trend=pop_age_trend, **popdict) # List for storing the people
 
     sc.printv(f'Created {n_agents} agents, average age {ages.mean():0.2f} years', 2, verbose)
 
@@ -165,8 +166,9 @@ def set_static(new_n, existing_n=0, pars=None, sex_ratio=0.5):
     debut           = np.full(new_n, np.nan, dtype=hpd.default_float)
     debut[sex==1]   = hpu.sample(**pars['debut']['m'], size=sum(sex))
     debut[sex==0]   = hpu.sample(**pars['debut']['f'], size=new_n-sum(sex))
+    rel_sev         = hpu.sample(**pars['sev_dist'], size=new_n) # Draw individual relative susceptibility factors
     partners        = partner_count(n_agents=new_n, partner_pars=pars['partners'])
-    return uid, sex, debut, partners
+    return uid, sex, debut, rel_sev, partners
 
 
 def validate_popdict(popdict, pars, verbose=True):
