@@ -11,13 +11,14 @@ from . import defaults as hpd
 from . import misc as hpm
 from . import base as hpb
 from . import sim as hps
+from . import utils as hpu
 from . import plotting as hppl
 from . import interventions as hpi
 from .settings import options as hpo
 
 
 # Specify all externally visible functions this file defines
-__all__ = ['make_metapars', 'MultiSim', 'Scenarios', 'single_run', 'multi_run', 'parallel']
+__all__ = ['make_metapars', 'MultiSim', 'Scenarios', 'Sweep', 'single_run', 'multi_run', 'parallel']
 
 
 
@@ -262,7 +263,10 @@ class MultiSim(hpb.FlexPretty):
 
         totalkeys = reduced_sim.result_keys('total')
         genotypekeys = reduced_sim.result_keys('genotype')
-        sexkeys = reduced_sim.result_keys('by_sex')
+        sexkeys = reduced_sim.result_keys('sex')
+        agekeys = reduced_sim.result_keys('age')
+        type_dist_keys = reduced_sim.result_keys('type_dist')
+        n_age_bins = len(reduced_sim['age_bins'])-1
 
         for reskey in totalkeys:
             raw[reskey] = np.zeros((reduced_sim.res_npts, len(self.sims)))
@@ -279,8 +283,18 @@ class MultiSim(hpb.FlexPretty):
             for s,sim in enumerate(self.sims):
                 vals = sim.results[reskey].values
                 raw[reskey][:, :, s] = vals
+        for reskey in agekeys:
+            raw[reskey] = np.zeros((n_age_bins, reduced_sim.res_npts, len(self.sims)))
+            for s,sim in enumerate(self.sims):
+                vals = sim.results[reskey].values
+                raw[reskey][:, :, s] = vals
+        for reskey in type_dist_keys:
+            raw[reskey] = np.zeros((reduced_sim['n_genotypes'], reduced_sim.res_npts, len(self.sims)))
+            for s,sim in enumerate(self.sims):
+                vals = sim.results[reskey].values
+                raw[reskey][:, :, s] = vals
 
-        for reskey in totalkeys + genotypekeys + sexkeys:
+        for reskey in totalkeys + genotypekeys + sexkeys + agekeys + type_dist_keys:
             if reskey in totalkeys:
                 axis = 1
                 results = reduced_sim.results
@@ -298,7 +312,6 @@ class MultiSim(hpb.FlexPretty):
                 results[reskey].low = np.quantile(raw[reskey], q=quantiles['low'], axis=axis)
                 results[reskey].high = np.quantile(raw[reskey], q=quantiles['high'], axis=axis)
 
-
         # Deal with analyzers
         reduced_analyzers = []
         for ai,analyzer in enumerate(reduced_sim['analyzers']): # Loop over each type of analyzer
@@ -999,12 +1012,16 @@ class Scenarios(hpb.ParsObj):
 
         totalkeys   = self.result_keys('total')
         genotypekeys = self.result_keys('genotype')
-        sexkeys = self.result_keys('by_sex')
+        sexkeys = self.result_keys('sex')
+        agekeys = self.result_keys('age')
+        type_dist_keys = self.result_keys('type_dist')
 
         # Loop over scenarios
         for scenkey,scen in self.scenarios.items():
             scenname = scen['name']
             scenpars = scen['pars']
+
+
             allpars = sc.mergedicts(scenpars, {'location': self.base_sim['location']})
 
             # This is necessary for plotting, and since self.npts is defined prior to run
@@ -1036,6 +1053,10 @@ class Scenarios(hpb.ParsObj):
             # Initialize the sim, including potentially the people and initial states
             scen_sim.initialize(reset=reset_people, init_states=reset_init_states)
 
+            if 'hiv_pars' in scen.keys():
+                hiv_scenpars = {'hiv_pars': sc.mergedicts(scen_sim.hivsim.pars['hiv_pars'], scen['hiv_pars'])}
+                scen_sim.hivsim.update_pars(hiv_scenpars)
+
             run_args = dict(n_runs=self['n_runs'], noise=self['noise'], noisepar=self['noisepar'], keep_people=keep_people, verbose=verbose)
             if debug:
                 print('Running in debug mode (not parallelized)')
@@ -1047,12 +1068,13 @@ class Scenarios(hpb.ParsObj):
             # Process the simulations
             print_heading(f'Processing {scenkey}')
             ng = scen_sims[0]['n_genotypes'] # Get number of genotypes
+            na = len(scen_sims[0]['age_bins'])-1 # Get number of age bins
             scenraw = {}
             for reskey in totalkeys:
                 scenraw[reskey] = np.zeros((self.res_npts, len(scen_sims)))
                 for s,sim in enumerate(scen_sims):
                     scenraw[reskey][:,s] = sim.results[reskey].values
-            for reskey in genotypekeys:
+            for reskey in genotypekeys+type_dist_keys:
                 scenraw[reskey] = np.zeros((ng, self.res_npts, len(scen_sims)))
                 for s,sim in enumerate(scen_sims):
                     scenraw[reskey][:,:,s] = sim.results[reskey].values
@@ -1060,6 +1082,10 @@ class Scenarios(hpb.ParsObj):
                 scenraw[reskey] = np.zeros((2, self.res_npts, len(scen_sims)))
                 for s,sim in enumerate(scen_sims):
                     scenraw[reskey][:,:,s] = sim.results[reskey].values
+            for reskey in agekeys:
+                scenraw[reskey] = np.zeros((na, self.res_npts, len(scen_sims)))
+                for s,sim in enumerate(scen_sims):
+                    scenraw[reskey][:,:,s] = sim.results[reskey].values
 
             scenres = sc.objdict()
             scenres.best = {}
@@ -1115,7 +1141,7 @@ class Scenarios(hpb.ParsObj):
         # Compute dataframe
         x = defaultdict(dict)
         genotypekeys = self.result_keys('genotype')
-        sexkeys = self.result_keys('by_sex')
+        sexkeys = self.result_keys('sex')
         totalkeys = self.result_keys('total')
         for scenkey in self.scenarios.keys():
             for reskey in self.result_keys():
@@ -1427,6 +1453,210 @@ class Scenarios(hpb.ParsObj):
         return scens
 
 
+class Sweep(MultiSim):
+    '''
+    Class for running parameter sweeps.
+
+    Args:
+        base_sim  (Sim)      : the sim used for shared properties
+        label      (str)     : the name of the multisim
+        kwargs    (dict)     : stored in run_args and passed to run()
+
+    Returns:
+        sweep: a Sweep object
+
+    **Examples**:
+        sim = hpv.Sim() # Create the sim
+        sweep = hpv.Sweep(sim, sweep_pars={'beta':[0.4, 0.8]}, n_runs=5) # Create the sweep
+        sweep.run() # Run the sims in parallel
+        sweep.plot() # Plot results
+    '''
+
+    def __init__(self, base_sim=None, sweep_pars=None, sweep_vars=None, n_draws=4, initialize=False, label=None,
+                 **kwargs):
+
+        # Set properties
+        self.base_sim   = base_sim
+        self.sweep_pars = sweep_pars
+        self.sweep_par_keys = list(sweep_pars.keys())
+        self.sweep_vars = sweep_vars
+        self.res_labels = None # Populated after sims are run
+        self.n_draws    = n_draws
+        self.from_year  = None # Populated within reduce()
+        self.to_year    = None # Populated within reduce()
+        self.label      = base_sim.label if (label is None) else label
+        self.run_args   = sc.mergedicts(kwargs)
+        self.resdf      = None
+        hpb.set_metadata(self) # Set version, date, and git info
+
+        # Create parameter draws
+        self.sweep_draws = sc.objdict()
+        for pname, prange in self.sweep_pars.items():
+            self.sweep_draws[pname] = np.random.uniform(prange[0], prange[1], self.n_draws)
+
+        # Create sims and optionally initialize
+        self.create_sims(initialize=initialize)
+
+        return
+
+    def create_sims(self, initialize=False):
+        self.sims = sc.autolist()
+        for ipd in range(self.n_draws):
+            sim = sc.dcp(self.base_sim)
+            for pname,pdraws in self.sweep_draws.items():
+                sim.pars[pname] = pdraws[ipd]
+            self.sims += sim
+
+        # Optionally initialize the sims
+        if initialize:
+            kwargs = sc.mergedicts(self.run_args, kwargs, {'do_run':False})
+            self.sims = multi_run(sims, **kwargs)
+
+
+    def run(self, shrink=True, reduce=True, sum_totals=True, from_year=None, to_year=None, **kwargs):
+        '''
+        Run the sims
+        Args:
+            shrink  (bool): whether or not to shrink after running
+            kwargs  (dict): passed to multi_run(); use run_args to pass arguments to sim.run()
+
+        Returns:
+            None (modifies MultiSim object in place)
+
+        **Examples**::
+            sweep.run()
+        '''
+
+        # Handle which sims to use
+        if self.sims is None:
+            sims = self.base_sim
+        else:
+            sims = self.sims
+
+            # Handle missing labels
+            for s,sim in enumerate(sims):
+                if sim.label is None:
+                    sim.label = f'Sim {s}'
+
+        # Run
+        kwargs = sc.mergedicts(self.run_args, kwargs)
+        self.sims = multi_run(sims, **kwargs)
+
+        # Shrink
+        if shrink:
+            self.shrink()
+
+        # Create results
+        if reduce:
+            self.reduce(sum_totals=sum_totals, from_year=from_year, to_year=to_year)
+
+        return self
+
+
+    def reduce(self, sum_totals=True, from_year=None, to_year=None):
+        ''' Create a dataframe holding reduced results '''
+
+        # Handle from and to years
+        if from_year is not None:
+            try:
+                i0 = sc.findinds(self.sims[0].res_yearvec, from_year)[0]
+            except:
+                errormsg = f'Cannot sum from {from_year}, summing from simulation start year instead'
+                print(errormsg)
+                from_year = self.sims[0].res_yearvec[0]
+                i0 = None
+            self.from_year = from_year
+        else:
+            i0 = None
+            self.from_year = self.sims[0].res_yearvec[0]
+
+        if to_year is not None:
+            try:
+                i1 = sc.findinds(self.sims[0].res_yearvec, to_year)[0]
+            except:
+                errormsg = f'Cannot sum to {to_year}, summing to simulation end year instead'
+                print(errormsg)
+                to_year = self.sims[0].res_yearvec[-1]
+                i1 = None
+        else:
+            i1 = None
+            self.to_year = self.sims[0].res_yearvec[-1]
+
+        # Store result labels
+        self.res_labels = {}
+        ref_sim = self.sims[0]
+        for svar in self.sweep_vars:
+            self.res_labels[svar] = ref_sim.results[svar].name
+
+        # Start creating result dataframe
+        dfts = sc.autolist()
+        for sim in self.sims:
+            dft = pd.DataFrame()
+            for spar in self.sweep_pars:
+                dft[spar] = [sim[spar]]
+            for svar in self.sweep_vars:
+                if sum_totals:
+                    dft[svar] = [sim.results[svar][i0:i1].sum()]
+                else:
+                    dft[svar] = sim.results[svar][i0:i1]
+            dfts += dft
+        self.resdf = pd.concat(dfts, ignore_index=True)
+
+        return
+
+    def combine(self, **kwargs):
+        raise NotImplementedError
+
+    def mean(self, **kwargs):
+        raise NotImplementedError
+
+    def median(self, **kwargs):
+        raise NotImplementedError
+
+    def compare(self, **kwargs):
+        raise NotImplementedError
+
+    def plot_heatmap(self, to_plot=None, xpar=None, ypar=None, zscales=1, npts=100, **kwargs):
+
+        # Handle plotting inputs
+        if to_plot is None: to_plot = self.sweep_vars
+        if type(to_plot)==str: to_plot = [to_plot]
+        if type(to_plot)==list:
+            for res_to_plot in to_plot:
+                if res_to_plot not in self.sweep_vars:
+                    errormsg = f'Cannot plot result {res_to_plot} as it does not appear in the stored results: {self.sweep_vars}.'
+                    raise ValueError(errormsg)
+
+        # Handle x and y parameters
+        if len(self.sweep_par_keys)<2:
+            raise ValueError(f'Cannot plot a heatmap without 2 or more sweep parameters: sweep_pars={self.sweep_par_keys}.')
+        else:
+            if xpar is None: xpar = self.sweep_par_keys[0]
+            else:
+                if xpar not in self.sweep_par_keys:
+                    errormsg = f'Cannot plot results for parameter {xpar} as it was not one of the sweep parameters: {self.sweep_par_keys}.'
+                    raise ValueError(errormsg)
+
+            if ypar is None: ypar = self.sweep_par_keys[1]
+            else:
+                if ypar not in self.sweep_par_keys:
+                    errormsg = f'Cannot plot results for parameter {ypar} as it was not one of the sweep parameters: {self.sweep_par_keys}.'
+                    raise ValueError(errormsg)
+
+        # Other validation
+        if self.resdf is None:
+            raise ValueError('Sweeps not run yet.')
+
+        # Create xy grid
+        x = np.array(self.resdf[xpar])
+        y = np.array(self.resdf[ypar])
+        xi = np.linspace(self.sweep_pars[xpar][0], self.sweep_pars[xpar][1], npts)
+        yi = np.linspace(self.sweep_pars[ypar][0], self.sweep_pars[ypar][1], npts)
+        xx, yy = np.meshgrid(xi, yi)
+
+        # Call plotting code to create and return figures
+        return hppl.plot_heatmap(self, xx, yy, x=x, y=y, xi=xi, yi=yi, to_plot=to_plot, xpar=xpar, ypar=ypar, zscales=zscales, npts=npts, **kwargs)
+
 
 def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, keep_people=False, run_args=None, sim_args=None, verbose=None, do_run=True, **kwargs):
     '''
@@ -1467,7 +1697,7 @@ def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, keep_people=Fa
 
     if reseed:
         sim['rand_seed'] += ind # Reset the seed, otherwise no point of parallel runs
-        sim.set_seed()
+        hpu.set_seed()
 
     # If the noise parameter is not found, guess what it should be
     if noisepar is None:
