@@ -80,19 +80,62 @@ def invlogf1(y, k):
 def logf2(x, x_infl, k):
     '''
     Logistic function, constrained to pass through 0,0 and with upper asymptote
-    at 1. Accepts 2 parameters: growth rate and point of inflexion.
+    at 1. Accepts 2 parameters: growth rate and point of inflection.
     '''
     l_asymp = -1/(1+np.exp(k*x_infl))
     return l_asymp + 1/( 1 + np.exp(-k*(x-x_infl)))
 
 
+def get_asymptotes(x_infl, k, ttc=25, s=1):
+    term1 = (1 + np.exp(k*(x_infl-ttc)))**s
+    term2 = (1 + np.exp(k*x_infl))**s
+    u_asymp_num = term1*(1-term2)
+    u_asymp_denom = term1 - term2
+    u_asymp = u_asymp_num / u_asymp_denom
+    l_asymp = term1 / (term1 - term2)
+    return l_asymp, u_asymp
+
+def logf3(x, x_infl, k, ttc=25, s=1):
+    l_asymp, u_asymp = get_asymptotes(x_infl, k, ttc, s)
+    return np.minimum(1, l_asymp + (u_asymp-l_asymp)/(1+np.exp(k*(x_infl-x)))**s)
+
+
+def invlogf3(y, x_infl, k, ttc=25, s=1):
+    l_asymp, u_asymp = get_asymptotes(x_infl, k, ttc, s)
+    part1 = np.log((u_asymp-l_asymp)/(y-l_asymp))/s
+    part2 = np.log(np.exp(part1)-1)
+    final = 1/k * (k*x_infl - part2)
+    return final
+
+
 def invlogf2(y, x_infl, k):
     '''
     Inverse logistic function, constrained to pass through 0,0 and with upper asymptote
-    at 1. Accepts 2 parameters: growth rate and point of inflexion.
+    at 1. Accepts 2 parameters: growth rate and point of inflection.
     '''
     l_asymp = -1/(1+np.exp(k*x_infl))
-    return (-1/k)*np.log((1/(y - l_asymp)) - 1) + x_infl
+    val = (1/(y - l_asymp)) - 1
+    if (val < 0).any():
+        val[true(val < 0)] = 0.001
+        # raise ValueError
+    result = (-1/k)*np.log(val) + x_infl
+    return result
+
+
+def transform_prob(tp,dysp):
+    '''
+    Returns transformation probability given % of dysplastic cells
+    '''
+
+    return 1-np.power(1-tp, dysp*100)
+
+
+def clearance_prob(init_clearance_prob, clearance_decay, dysp):
+    '''
+    Returns clearance probability given % of transformed cells
+    '''
+
+    return init_clearance_prob*(1-(1 - np.power(1 - clearance_decay, dysp * 100)))
 
 
 #%% Sampling and seed methods
